/*
 * Custom.java
 *
 * Created on 30 mai 2007, 05:06
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.library.jiga.sf3.game;

import java.awt.Dimension;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * The Custom class extends the abstract class SpritesChar to implement customized animation models. Each animation listed can be linked to a String value.
 * It defines the paths to animation and sound files, as well as the various properties.
 * @see #animsMap
 * @see #sfxMap
 * @author website
 */
public class Custom extends SpritesChar implements Serializable{
    /** Maps of the animations and sound effects, all linked to animations ids
     * @see SpritesChar#animsID*/
    Map<String, Integer> animsMap, sfxMap;
    /** creates a customized character or Model
     * @param name character's id or base path
     * @param res common resolution ID of the animation sprites files. It is defined in the super class.
     *@see SpritesChar#RES_EXT_map
     * @param floorDiff the actual floor difference in pixels from the first pixel to define as floor and the sprite bottom (height)
     * @see SpritesChar#floorDiff
     * @param spriteDim actual size of the sprites
     * @param spriteSize average inset size of the sprites
     * @param framesNum 2D-array of the frames number to access sprites files (x : first frame inclusive, y : last frame inclusive)
     * @param reversedAnims 1D-array of the ids (String values) of the animations of sprites to be playing-reversed
     * @see #animsMap
     * @param sfxRsrcs 1D-array of the filenames of the sound effects used with this character (base path relative)*/
    Custom(String name, int res, int floorDiff, Dimension spriteDim, Dimension spriteSize, int[][] framesNum, String[] reversedAnims, String[] sfxRsrcs) {
        this(res);
        id = name;
        this.floorDiff = (int)((float)floorDiff / RES_FACTORS_map.get(res).floatValue());
        spriteWidth = (int)((float)spriteDim.width / RES_FACTORS_map.get(res).floatValue());
        spriteHeight = (int)((float)spriteDim.height/ RES_FACTORS_map.get(res).floatValue());
        this.spriteSize = new Dimension(
                (int)((float)spriteSize.width / RES_FACTORS_map.get(res).floatValue()),
                (int)((float)spriteSize.height / RES_FACTORS_map.get(res).floatValue()));
        frames = framesNum;
        this.reversedAnims = reversedAnims;
        sfx = sfxRsrcs;
    }
    
    /** creates a customized character or Model
      @param res common resolution ID of the animation sprites files. It is defined in the super class.
     *@see SpritesChar#RES_EXT_map
     */
    Custom(int res) {
        super();
        this.res = res;
        sfxMap = Collections.synchronizedMap(new HashMap<String, Integer>());
        animsMap = Collections.synchronizedMap(new HashMap<String,Integer>());
    }
    
    /** adds a new mapping to the animations that are referenced by the values of this mapping
     * @see #anim(String)
     * @see #animsMap
     @param anims the mapping to add*/
    public void addAnimMap(Map<String, Integer> anims) {
        animsMap.putAll(anims);
    }
    
    /** adds a new mapping to the sound effects that are reference by the values of this mapping
     * @see #sfxMap
     @see #sound(String)*/
    public void addSoundMap(Map<String, Integer> sfx) {
        sfxMap.putAll(sfx);
    }
    
    /** adds a new mapping to the animations that are referenced by the values of this mapping
     * @param name the String value to link with the animation
     @param index the index value to get the animation from the current class-map
     @see #animsMap
     @see #anim(String)*/
    public void addAnimMap(String name, int index) {
        animsMap.put(name, index);
    }
    
    /** adds a new mapping to the sounds that are referenced by the values of this mapping
     * @param name the String value to link with the sound
     @param index the index value to get the sound from the current class-map
     @see #sfxMap
     @see #sound(String)*/
    public void addSoundMap(String name, int index) {
        sfxMap.put(name, index);
    }
    
    /** returns the linked index value linked to this String value referencing a mapped animation in this class
     @param name the String value referencing a mapped animation
     @return the index value of the requested animation*/
    public int anim(String name) {
        return (animsMap.containsKey(name))?animsMap.get(name):0;
    }
    
    /** gives the requested animation playing-reversed status
     @return true or false*/
    public boolean isReversedAnim(String name) {
        for(int i = 0; i < reversedAnims.length; i++) {
            if(reversedAnims[i].equals(name))
                return true;
        }
        return false;
    }
    
/** returns the linked index value linked to this String value referencing a mapped sound in this class
     @param name the String value referencing a mapped sound
     @return the index value of the requested sound*/
    public int sound(String name) {
        return (sfxMap.containsKey(name))?sfxMap.get(name):0;
    }
    
    /** defines the prefix String to all sprite files
     @param prefix the String prefix to use*/
    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }
    
    /** returns the current prefix String for all sprite files
     @return the current prefix String*/
    public String getPrefix() {
        return prefix;
    }
    
    /** defines the suffix String to all sprite files
     @param suffix the String suffix to use*/
    public void setSuffix(String suffix) {
        this.suffix = suffix;
    }
    
    /** returns the current suffix String for all sprite files
     @return the current suffix String*/
    public String getSuffix() {
        return suffix;
    }
}
